//--------------------------------------------------------------
// File     : stm32_ub_ov9655.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_OV9655_H
#define __STM32F4_UB_OV9655_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_dcmi.h"
#include "stm32f4xx_dma.h"
#include "stm32_ub_i2c1.h"
#include "stm32_ub_lcd_st7783.h"


//--------------------------------------------------------------
// Return Werte beim initialisieren
//--------------------------------------------------------------
typedef enum {
  OV9655_OK =0,    // OV9655 OK
  OV9655_I2C_ERR,  // Fehler bei I2C
  OV9655_MIDH_ERR, // Fehler bei ID-HIByte
  OV9655_MIDL_ERR, // Fehler bei ID-LoByte
  OV9655_PID_ERR,  // Fehler bei Produkt-ID
  OV9655_LCD_ERR   // Fehler beim Display
}OV9655_ERR_t;


//--------------------------------------------------------------
// Camera-Mode
// Mode-0 :  QVGA = 320x240 Pixel
// Mode-1 : QQVGA = 160x120 Pixel
//--------------------------------------------------------------
#define  OV9655_AKT_MODE     0  // 320x240 Pixel
//#define  OV9655_AKT_MODE     1  // 160x120 Pixel





//--------------------------------------------------------------
// RAM-Snapshot-Mode
//--------------------------------------------------------------
#define  USE_OV9655_RAM_SNAPSHOT   0 // 0=ohne RAM Buffer
//#define  USE_OV9655_RAM_SNAPSHOT   1 // 1=mit RAM Buffer







//--------------------------------------------------------------
//  RAM-Snapshot nur im QQVGA-Mode !!
//--------------------------------------------------------------
#if OV9655_AKT_MODE==0
  #if USE_OV9655_RAM_SNAPSHOT==1
    #error RAM_SNAPSHOP dont work in QVGA-Mode
  #endif
#endif




//--------------------------------------------------------------
// Ziel-Adresse der Camera-Daten
//--------------------------------------------------------------
#define  OV9655_DCMI_MEMORY_ADR   LCD_RAM_ADR  // LCD-Ram




//--------------------------------------------------------------
// Auflsung
//--------------------------------------------------------------
#if OV9655_AKT_MODE==0
  #define OV9655_MAXX   320
  #define OV9655_MAXY   240
  #define OV9655_PIXEL  OV9655_MAXX*OV9655_MAXY
#else
  #define OV9655_MAXX   160
  #define OV9655_MAXY   120
  #define OV9655_PIXEL  OV9655_MAXX*OV9655_MAXY
#endif


//--------------------------------------------------------------
// I2C Daten vom OV9655
// max I2C-Frq = 400kHz
//--------------------------------------------------------------
#define  OV9655_I2C_ADR        0x60  // Slave-Adresse vom OV9655


#define  OV9655_I2C_REG_PID    0x0A  // Produkt-ID (= 0x96)
#define  OV9655_I2C_REG_COM7   0x12  // fr Reset
#define  OV9655_I2C_REG_MIDH   0x1C  // ID-HIByte  (= 0x7F)
#define  OV9655_I2C_REG_MIDL   0x1D  // ID-LoByte  (= 0xA2)
#define  OV9655_I2C_REG_BRTN   0x55  // Helligkeit


#define  OV9655_I2C_WR_PAUSE   0xFF    // kleine Pause beim write
#define  OV9655_I2C_RES_PAUSE  0xFFFF  // kleine Pause nach Reset


//--------------------------------------------------------------
// Adresse vom DCMI
// (siehe Seite 53 vom Referenz Manual)
//--------------------------------------------------------------
#define OV9655_DCMI_BASE_ADR        ((uint32_t)0x50050000)


//--------------------------------------------------------------
// Adressen der DCMI-Register
// (siehe Seite 347 vom Referenz Manual)
//--------------------------------------------------------------
#define OV9655_DCMI_REG_DR_OFFSET   0x28  // DR-Register

#define OV9655_DCMI_REG_DR_ADDRESS  (OV9655_DCMI_BASE_ADR | OV9655_DCMI_REG_DR_OFFSET)



//--------------------------------------------------------------
// DMA Einstellung
// Moegliche DMAs fuer DCMI :
//   DMA2_STREAM1_CHANNEL1
//   DMA2_STREAM7_CHANNEL1
//--------------------------------------------------------------
 
#define OV9655_DCMI_DMA_STREAM1        1
//#define OV9655_DCMI_DMA_STREAM7        7


#ifdef OV9655_DCMI_DMA_STREAM1
  #define OV9655_DCMI_DMA_STREAM            DMA2_Stream1
  #define OV9655_DCMI_DMA_CHANNEL           DMA_Channel_1
#elif defined OV9655_DCMI_DMA_STREAM7
  #define OV9655_DCMI_DMA_STREAM            DMA2_Stream7
  #define OV9655_DCMI_DMA_CHANNEL           DMA_Channel_1
#endif



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
OV9655_ERR_t UB_OV9655_Init(void);
void UB_OV9655_CaptureStart(void);
void UB_OV9655_CaptureStop(void);
void UB_OV9655_Snapshot(void);
void UB_OV9655_SetHelligkeit(uint8_t wert);

#if USE_OV9655_RAM_SNAPSHOT==1
#define   BMP_HEADER_LEN    54
void UB_OV9655_Snapshot2RAM(void);
void UB_OV9655_RAM2BMP(void);
#endif




//--------------------------------------------------------------
#endif // __STM32F4_UB_OV9655_H
